@@ -222,7 +222,7 @@ TEST_CASE("Parsing coordinates from strings") {
     C("1.1234567",  11234567);
     C("1.12345670", 11234567);
     C("1.12345674", 11234567);
-    C("1.12345675", 11234568);
+    C("1.123456751", 11234568);
     C("1.12345679", 11234568);
     C("1.12345680", 11234568);
     C("1.12345681", 11234568);
@@ -233,6 +233,37 @@ TEST_CASE("Parsing coordinates from strings") {
     C("179.99999999", 1800000000);
     C("200.123",      2001230000);
 
+    C("1e2",   1000000000);
+    C("1e1",    100000000);
+    C("1e0",     10000000);
+    C("1e-1",     1000000);
+    C("1e-2",      100000);
+    C("1e-3",       10000);
+    C("1e-4",        1000);
+    C("1e-5",         100);
+    C("1e-6",          10);
+    C("1e-7",           1);
+
+    C("1.0e2",   1000000000);
+    C("1.1e1",    110000000);
+    C("0.1e1",     10000000);
+    C("1.2e0",     12000000);
+    C("1.9e-1",     1900000);
+    C("2.0e-2",      200000);
+    C("2.1e-3",       21000);
+    C("9.0e-4",        9000);
+    C("9.1e-5",         910);
+    C("1.0e-6",          10);
+    C("1.0e-7",           1);
+    C("1.4e-7",           1);
+    C("1.5e-7",           2);
+    C("1.9e-7",           2);
+
+    F("e");
+    F(" e");
+    F(" 1.1e2");
+    F("1.1e2 ");
+    F("1.1e2x");
 }
 
 #undef C
